// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2000-2004 - Benoit Hamelin, Jean-Pierre Dussault
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->
// <-- NOT FIXED -->


////////////////////////////////////////

// Returns the status of the optimization with respect to 
// the function value or the point X.
// 
function [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX )
  [foptE,xoptE] = uncprb_getopt(nprob,n,m);
  // Analyse the function value
  if ( abs(foptE) == 0 ) then
    if ( abs(foptC)< tolF ) then
      sF = "OK";
    else
      sF = "FAIL";
    end
  else
    if ( abs(foptC-foptE)/abs(foptE)< tolF ) then
      sF = "OK";
    else
      sF = "FAIL";
    end
  end
  // Analyse the point x
  if ( norm(xoptE) == 0 ) then
    if ( norm(xoptC-xoptE)< tolX ) then
      sX = "OK";
    else
      sX = "FAIL";
    end
  else
    if ( norm(xoptC-xoptE)/norm(xoptE)< tolX ) then
      sX = "OK";
    else
      sX = "FAIL";
    end
  end
endfunction

mprintf("\n==\n");
mprintf("Testing optim on several problems in this collection.\n");
mprintf("Press return to continue\n");
halt()
mprintf("\n==\n");

//
// Test optim on these problems
// Tolerance on function value
tolF = 1.e-8;
tolX = 1.e-8;
mprintf("==================================\n")
mprintf("Compares several approaches with optim:\n")
mprintf(" * optim unbounded with default parameters\n")
mprintf(" * optim unbounded with tuned parameters\n")
mprintf(" * optim with infinite bounds and default parameters\n")
mprintf(" * optim with infinite bounds and tuned parameters\n")
mprintf("==================================\n")
mprintf("Tolerance on F:%e\n",tolF)
mprintf("Tolerance on F:%e\n",tolF)
mprintf("Tolerances are relative when possible, absolute if not.\n")
mprintf("==================================\n")
mprintf("Testing optim with default settings\n")
for nprob = [1:19,23,25,26,28:34]
  [n,m,x0] = uncprb_getinitf(nprob);
  //
  // Define a wrapper over the objective function for the optimizer
  body = [
      "fout=uncprb_getobjfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
      "gout=uncprb_getgrdfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
  ];
  prot=funcprot();
  funcprot(0);
  deff("[fout,gout,ind]=objfun(x,ind)",body);
  funcprot(prot);
  //
  [foptC,xoptC,goptC]=optim(objfun,x0);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Problem #%d, fopt = %e, gopt = %e, F=%s, X=%s\n", nprob , foptC , norm(goptC), sF, sX)
end

// Test optim / bounded on these problems
mprintf("==================================\n")
mprintf("Testing optim with infinite bounds and default settings\n")
for nprob = [1:19,23,25,26,28:34]
  [n,m,x0] = uncprb_getinitf(nprob);
  //
  // Define a wrapper over the objective function for the optimizer
  body = [
      "fout=uncprb_getobjfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
      "gout=uncprb_getgrdfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
  ];
  prot=funcprot();
  funcprot(0);
  deff("[fout,gout,ind]=objfun(x,ind)",body);
  funcprot(prot);
  //
  [foptC,xoptC,goptC]=optim(objfun,"b",-%inf*ones(n,1),%inf*ones(n,1),x0);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Problem #%d, fopt = %e, gopt = %e, F=%s, X=%s\n", nprob , foptC , norm(goptC), sF, sX)
end

//
// Manual tuning for Problem #2
// The algorithm fails to converge with the default parameters.
// The gradient is small, but the point is away from the optimum and the function
// value is large.
// Both two optim algorithms fail.
nprob = 2;
mprintf("==================================\n")
mprintf("Manual tests for problem #%d\n",nprob)
  [n,m,x0] = uncprb_getinitf(nprob);
  //
  // Define a wrapper over the objective function for the optimizer
  body = [
      "fout=uncprb_getobjfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
      "gout=uncprb_getgrdfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
  ];
  prot=funcprot();
  funcprot(0);
  deff("[fout,gout,ind]=objfun(x,ind)",body);
  funcprot(prot);
  //
  [foptC,xoptC,goptC]=optim(objfun,x0,"ar",1000,1000);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-unbounded/tuned, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)
  //
  [foptC,xoptC,goptC]=optim(objfun,"b",-%inf*ones(n,1),%inf*ones(n,1),x0,"ar",1000,1000);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-unbounded/tuned, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)
  // The Hessian is not provided for this test: we use finite differences.
  Hfd = uncprb_gethesfd ( n , m , xoptC , nprob , [] , [] )
  // The conditionning is large, but not that much : 4.1D+04
  cond(Hfd)
//
// Manual tuning for Problem #3
// The algorithm fails to converge with the default parameters.
// The first component of the gradient is too large.
// It suffices to increase the number of iterations and function calls : 
// it converges in 150 iterations with 197 function calls.
nprob = 3;
mprintf("==================================\n")
mprintf("Manual tests for problem #%d\n",nprob)
  [n,m,x0] = uncprb_getinitf(nprob);
  //
  // Define a wrapper over the objective function for the optimizer
  body = [
      "fout=uncprb_getobjfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
      "gout=uncprb_getgrdfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
  ];
  prot=funcprot();
  funcprot(0);
  deff("[fout,gout,ind]=objfun(x,ind)",body);
  funcprot(prot);
  //
  [foptC,xoptC,goptC]=optim(objfun,x0);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-unbounded/default, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)
  //
  [foptC,xoptC,goptC]=optim(objfun,x0,"ar",1000,1000);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-unbounded/tuned, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)
//
// Manual tuning for Problem #7
// The algorithm fails to converge with the default parameters.
// Increasing the number of iterations or function calls does not improve.
// It suffices to use the bounded algorithm to make the algorithm work.
nprob = 7;
mprintf("==================================\n")
mprintf("Manual tests for problem #%d\n",nprob)
  [n,m,x0] = uncprb_getinitf(nprob);
  //
  // Define a wrapper over the objective function for the optimizer
  body = [
      "fout=uncprb_getobjfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
      "gout=uncprb_getgrdfcn("+string(n)+","+string(m)+",x,"+string(nprob)+")"
  ];
  prot=funcprot();
  funcprot(0);
  deff("[fout,gout,ind]=objfun(x,ind)",body);
  funcprot(prot);
  //
  [foptC,xoptC,goptC]=optim(objfun,x0,"ar",1000,1000);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-unbounded/default, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)
  [foptC,xoptC,goptC]=optim(objfun,"b",-%inf*ones(n,1),%inf*ones(n,1),x0);
  [sF, sX] = computestatus ( xoptC, foptC , nprob , tolF , tolX );
  mprintf("Optim-infinite-bounded/default, fopt = %e, gopt = %e, F=%s, X=%s\n", foptC , norm(goptC), sF, sX)

//
// Load this script into the editor
//
filename = "bench_optim.sce";
dname = get_absolute_file_path(filename);
editor ( dname + filename );

