// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->


//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
////////////////////////////////////////

// Get fvec and J at x0 for Rosenbrock's test case
n = 2;
m = 2;
x = [-1.2,1]';
option = 3;
nprob = 1;
[fvec,J]=uncprb_getfunc(n,m,x,nprob,option);
assert_close ( fvec , [-4.4 2.2]', %eps );
Je = [
    24.    10.  
  - 1.     0.   
];
assert_close ( J , Je, %eps );

// See what happens if option=1
n = 2;
m = 2;
x = [-1.2,1]';
option = 1;
nprob = 1;
[fvec,J]=uncprb_getfunc(n,m,x,nprob,option);
assert_close ( fvec , [-4.4 2.2]', %eps );
assert_equal ( J , [] );

// See what happens if option=2
n = 2;
m = 2;
x = [-1.2,1]';
option = 2;
nprob = 1;
[fvec,J]=uncprb_getfunc(n,m,x,nprob,option);
assert_equal ( fvec , [] );
Je = [
    24.    10.  
  - 1.     0.   
];
assert_close ( J , Je, %eps );


