// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->


//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
////////////////////////////////////////

// Get name for Rosenbrock's test case
name = uncprb_getname(1);
assert_equal ( name , "rose" );
name = uncprb_getname(1,%f);
assert_equal ( name , "rose" );
name = uncprb_getname(1,%t);
assert_equal ( name , "Rosenbrock" );

// Check that all names can be retrieved
nprobmax = uncprb_getproblems();
for nprob = 1 : nprobmax
  name = uncprb_getname(nprob);
  assert_equal ( typeof(name) , "string" );
  name = uncprb_getname(nprob,%t);
  assert_equal ( typeof(name) , "string" );
end

// See a case of error
cmd = "name = uncprb_getname(-1)";
execstr(cmd,"errcatch");
[str,errcode] = lasterror();
expected = "uncprb_getname: The input argument nprob is -1 but should be between 1 and 35.";
assert_equal ( str , expected );
//
// Get all names at once
nprobmax = uncprb_getproblems();
name = uncprb_getname(1:nprobmax);
assert_equal ( size(name) , [nprobmax 1] );

