// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->


//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
////////////////////////////////////////

// Use default
nprob = 1;
[n,m,x0]=uncprb_getinitf(nprob);
Jfd = uncprb_getvecjacfd ( n , m , x0 , nprob );
J = [
24 10
-1 0
];
assert_close ( Jfd , J , 1.e-10 );

// Use default
nprob = 1;
[n,m,x0]=uncprb_getinitf(nprob);
Jfd = uncprb_getvecjacfd ( n , m , x0 , nprob , [] , [] );
J = [
24 10
-1 0
];
assert_close ( Jfd , J , 1.e-10 );

// Set step
nprob = 1;
[n,m,x0]=uncprb_getinitf(nprob);
Jfd = uncprb_getvecjacfd ( n , m , x0 , nprob , 1.e-1 );
J = [
24 10
-1 0
];
assert_close ( Jfd , J , 1.e-15 );

// Set step and order
nprob = 1;
[n,m,x0]=uncprb_getinitf(nprob);
Jfd = uncprb_getvecjacfd ( n , m , x0 , nprob , 1.e-1 , 4 );
J = [
24 10
-1 0
];
assert_close ( Jfd , J , 1.e-14 );

// Set order (use default step)
nprob = 1;
[n,m,x0]=uncprb_getinitf(nprob);
Jfd = uncprb_getvecjacfd ( n , m , x0 , nprob , [] , 4 );
J = [
24 10
-1 0
];
assert_close ( Jfd , J , 1.e-12 );

